"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var defaultProtocols = [];
var defaultWebsocketFactory = function (url, protocols) {
    if (protocols === void 0) { protocols = defaultProtocols; }
    return new WebSocket(url, protocols);
};
function connect(url, input, protocols, websocketFactory) {
    if (protocols === void 0) { protocols = defaultProtocols; }
    if (websocketFactory === void 0) { websocketFactory = defaultWebsocketFactory; }
    var connectionStatus = new rxjs_1.BehaviorSubject(0);
    var messages = new rxjs_1.Observable(function (observer) {
        var socket = websocketFactory(url, protocols);
        var inputSubscription;
        var open = false;
        var forcedClose = false;
        var closed = function () {
            if (!open)
                return;
            connectionStatus.next(connectionStatus.getValue() - 1);
            open = false;
        };
        socket.onopen = function () {
            open = true;
            connectionStatus.next(connectionStatus.getValue() + 1);
            inputSubscription = input.subscribe(function (data) {
                socket.send(data);
            });
        };
        socket.onmessage = function (message) {
            observer.next(message.data);
        };
        socket.onerror = function (error) {
            closed();
            observer.error(new Error(error.message));
        };
        socket.onclose = function (event) {
            // prevent observer.complete() being called after observer.error(...)
            if (!open)
                return;
            closed();
            if (forcedClose)
                observer.complete();
            else
                observer.error(new Error(event.reason));
        };
        return function () {
            forcedClose = true;
            if (inputSubscription)
                inputSubscription.unsubscribe();
            if (open) {
                closed();
                socket.close();
            }
        };
    });
    return { messages: messages, connectionStatus: connectionStatus };
}
exports.default = connect;
//# sourceMappingURL=index.js.map